from conga_constants import LUCI_DEBUG_MODE, LUCI_DEBUG_VERBOSITY
from syslog import openlog, closelog, syslog, \
		LOG_DAEMON, LOG_PID, LOG_NDELAY, LOG_INFO, \
		LOG_WARNING, LOG_AUTH, LOG_DEBUG

"""Exception class for the LuciSyslog facility
"""
class LuciSyslogError(Exception):
	def __init__(self, msg):
		Exception.__init__(self, msg)

"""Facility that provides centralized syslog(3) functionality for luci
"""
class LuciSyslog:
	def __init__(self):
		self.__init = 0
		try:
			openlog('luci', LOG_PID | LOG_NDELAY, LOG_DAEMON)
			self.__init = 1
		except:
			raise LuciSyslogError, 'unable to setup syslog functionality.'

	def info(self, msg):
		if not self.__init:
			return
		try:
			syslog(LOG_INFO, msg)
		except:
			raise LuciSyslogError, 'syslog info call failed'

	def warn(self, msg):
		if not self.__init:
			return
		try:
			syslog(LOG_WARNING, msg)
		except:
			raise LuciSyslogError, 'syslog warn call failed'

	def private(self, msg):
		if not self.__init:
			return
		try:
			syslog(LOG_AUTH, msg)
		except:
			raise LuciSyslogError, 'syslog private call failed'

	def debug_verbose(self, msg):
		if not LUCI_DEBUG_MODE or LUCI_DEBUG_VERBOSITY < 2 or not self.__init:
			return
		try:
			syslog(LOG_DEBUG, msg)
		except:
			raise LuciSyslogError, 'syslog debug call failed'

	def debug(self, msg):
		if not LUCI_DEBUG_MODE or not self.__init:
			return
		try:
			syslog(LOG_DEBUG, msg)
		except:
			raise LuciSyslogError, 'syslog debug call failed'

	def close(self):
		try:
			closelog()
		except:
			pass
