import xml
from time import time, ctime
from xml.dom import minidom
from ricci_communicator import RicciCommunicator
from LuciSyslog import LuciSyslog

try:
	luci_log = LuciSyslog()
except:
	pass

def checkBatch(rc, batch_id):
	try:
		batch = rc.batch_report(batch_id)
		if batch is None:
			return True
	except:
		return False

	try:
		batchid = batch.getAttribute('batch_id')
		result = batch.getAttribute('status')
	except:
		return False

	if result == '0':
		return True

	return False

def addClusterNodeBatch(cluster_name,
						install_base,
						install_services,
						install_shared_storage,
						install_LVS,
						upgrade_rpms):

	batch = '<?xml version="1.0" ?>'
	batch += '<batch>'
	batch += '<module name="rpm">'
	batch += '<request API_version="1.0">'
	batch += '<function_call name="install">'
	batch += '<var name="upgrade" type="boolean" value="'
	if upgrade_rpms:
		batch += 'true'
	else:
		batch += 'false'
	batch += '"/>'

	batch += '<var name="sets" type="list_xml">'

	if install_base or install_services or install_shared_storage:
		batch += '<set name="Cluster Base"/>'
	if install_services:
		batch += '<set name="Cluster Service Manager"/>'
	if install_shared_storage:
		batch += '<set name="Clustered Storage"/>'
	if install_LVS:
		batch += '<set name="Linux Virtual Server"/>'

	batch += '</var>'
	batch += '</function_call>'
	batch += '</request>'
	batch += '</module>'
		
	need_reboot = install_base or install_services or install_shared_storage or install_LVS
	if need_reboot:
		batch += '<module name="reboot">'
		batch += '<request API_version="1.0">'
		batch += '<function_call name="reboot_now"/>'
		batch += '</request>'
		batch += '</module>'
	else:
		# need placeholder instead of reboot
		batch += '<module name="rpm">'
		batch += '<request API_version="1.0">'
		batch += '<function_call name="install"/>'
		batch += '</request>'
		batch += '</module>'
		
	batch += '<module name="cluster">'
	batch += '<request API_version="1.0">'
	batch += '<function_call name="set_cluster.conf">'
	batch += '<var mutable="false" name="propagate" type="boolean" value="false"/>'
	batch += '<var mutable="false" name="cluster.conf" type="xml">'
	batch += '<cluster config_version="1" name="' + cluster_name + '">'
	batch += '<fence_daemon post_fail_delay="0" post_join_delay="3"/>'
	batch += '<clusternodes/>'
	batch += '<cman/>'
	batch += '<fencedevices/>'
	batch += '<rm/>'
	batch += '</cluster>'
	batch += '</var>'
	batch += '</function_call>'
	batch += '</request>'
	batch += '</module>'

	batch += '<module name="cluster">'
	batch += '<request API_version="1.0">'
	batch += '<function_call name="start_node"/>'
	batch += '</request>'
	batch += '</module>'
	batch += '</batch>'

	return minidom.parseString(batch).firstChild

def createClusterBatch(	os_str,
						cluster_name,
						cluster_alias,
						nodeList,
						install_base,
						install_services,
						install_shared_storage,
						install_LVS,
						upgrade_rpms):

	if os_str == 'rhel5':
		cluster_version = '5'
	elif os_str == 'rhel4':
		cluster_version = '4'
	else:
		cluster_version = 'unknown'

	batch = '<?xml version="1.0" ?>'
	batch += '<batch>'
	batch += '<module name="rpm">'
	batch += '<request API_version="1.0">'
	batch += '<function_call name="install">'
	batch += '<var name="upgrade" type="boolean" value="'
	if upgrade_rpms:
		batch += 'true'
	else:
		batch += 'false'
	batch += '"/>'
	batch += '<var name="sets" type="list_xml">'

	if install_base or install_services or install_shared_storage:
		batch += '<set name="Cluster Base"/>'
	if install_services:
		batch += '<set name="Cluster Service Manager"/>'
	if install_shared_storage:
		batch += '<set name="Clustered Storage"/>'
	if install_LVS:
		batch += '<set name="Linux Virtual Server"/>'
	batch += '</var>'
	batch += '</function_call>'
	batch += '</request>'
	batch += '</module>'

	need_reboot = install_base or install_services or install_shared_storage or install_LVS
	if need_reboot:
		batch += '<module name="reboot">'
		batch += '<request API_version="1.0">'
		batch += '<function_call name="reboot_now"/>'
		batch += '</request>'
		batch += '</module>'
	else:
		# need placeholder instead of reboot
		batch += '<module name="rpm">'
		batch += '<request API_version="1.0">'
		batch += '<function_call name="install"/>'
		batch += '</request>'
		batch += '</module>'

	batch += '<module name="cluster">'
	batch += '<request API_version="1.0">'
	batch += '<function_call name="set_cluster.conf">'
	batch += '<var mutable="false" name="propagate" type="boolean" value="false"/>'
	batch += '<var mutable="false" name="cluster.conf" type="xml">'
	batch += '<cluster config_version="1" name="' + cluster_name + '" alias="' + cluster_alias + '">'
	batch += '<fence_daemon post_fail_delay="0" post_join_delay="3"/>'

	batch += '<clusternodes>'

	x = 1
	for i in nodeList:
		if os_str == "rhel4":
			batch += '<clusternode name="' + i + '" votes="1" />'
		else:
			batch += '<clusternode name="' + i + '" votes="1" nodeid="' + str(x) + '" />'
		x = x + 1

	batch += '</clusternodes>'

	if len(nodeList) == 2:
		batch += '<cman expected_votes="1" two_node="1"/>'
	else:
		batch += '<cman/>'
 
	batch += '<fencedevices/>'
	batch += '<rm/>'
	batch += '</cluster>'
	batch += '</var>'
	batch += '</function_call>'
	batch += '</request>'
	batch += '</module>'

	batch += '<module name="cluster">'
	batch += '<request API_version="1.0">'
	batch += '<function_call name="start_node">'
	batch += '<var mutable="false" name="cluster_startup" type="boolean" value="true"/>'
	batch += '</function_call>'
	batch += '</request>'
	batch += '</module>'
	batch += '</batch>'

	return minidom.parseString(batch).firstChild

def batchAttemptResult(doc):
	try:
		batch = doc.getElementsByTagName('batch')
		if not batch or len(batch) < 1:
			raise Exception, 'no batch tag was found'
	except Exception, e:
		luci_log.debug_verbose('batchAttemptResult: %s' % str(e))

	for i in batch:
		try:
			batch_number = i.getAttribute('batch_id')
			result = i.getAttribute('status')
			return (str(batch_number), str(result))
		except Exception, e:
			luci_log.debug_verbose('batchAttemptResult: %s' % str(e))

	try:
		luci_log.debug_verbose('no batch with batchid and status found in \"%s\"' % doc.toxml())
	except:
		pass

	return (None, None)
	

def getPayload(bt_node):
	if not bt_node:
		return None

	mod_node = None
	for node in bt_node.childNodes:
		if node.nodeType == xml.dom.Node.ELEMENT_NODE and node.nodeName == 'module':
			mod_node = node
	if not mod_node:
		return None

	resp_node = None
	for node in mod_node.childNodes:
		if node.nodeType == xml.dom.Node.ELEMENT_NODE:
			resp_node = node
	if not resp_node:
		return None

	fr_node = None
	for node in resp_node.childNodes:
		if node.nodeType == xml.dom.Node.ELEMENT_NODE:
			fr_node = node
	if not fr_node:
		return None

	varnode = None
	for node in fr_node.childNodes:
		if node.nodeName == 'var':
			varnode = node
			break
	if not varnode:
		return None

	cl_node = None
	for node in varnode.childNodes:
		if node.nodeName == 'cluster':
			cl_node = node
			break
	if not cl_node:
		return None

	doc = minidom.Document()
	doc.appendChild(cl_node)
	return doc

def getClusterStatusBatch(rc):
	batch_str ='<module name="cluster"><request API_version="1.0"><function_call name="status"/></request></module>'
	ricci_xml = rc.batch_run(batch_str, async=False)

	if not ricci_xml or not ricci_xml.firstChild:
		luci_log.debug_verbose('ricci_xml is None from batch_run')
		
	doc = getPayload(ricci_xml.firstChild)
	if not doc or not doc.firstChild:
		luci_log.debug_verbose('doc is None from getPayload: %s' % ricci_xml.toxml())
		return None

	return doc

def setClusterConf(rc, clusterconf, propagate=True):
	if propagate == True:
		propg = 'true'
	else:
		propg = 'false'

	conf = str(clusterconf).replace('<?xml version="1.0"?>', '')
	conf = conf.replace('<?xml version="1.0" ?>', '')
	conf = conf.replace('<? xml version="1.0"?>', '')
	conf = conf.replace('<? xml version="1.0" ?>', '')

	batch_str = '<module name="cluster"><request API_version="1.0"><function_call name="set_cluster.conf"><var type="boolean" name="propagate" mutable="false" value="' + propg + '"/><var type="xml" mutable="false" name="cluster.conf">' + conf + '</var></function_call></request></module>'

	ricci_xml = rc.batch_run(batch_str)
	return batchAttemptResult(ricci_xml)

def getNodeLogs(rc):
	errstr = 'log not accessible'

	batch_str = '<module name="log"><request sequence="1254" API_version="1.0"><function_call name="get"><var mutable="false" name="age" type="int" value="18000"/><var mutable="false" name="tags" type="list_str"></var></function_call></request></module>'

	ricci_xml = rc.batch_run(batch_str, async=False)
	if not ricci_xml:
		return errstr
	try:
		log_entries = ricci_xml.getElementsByTagName('logentry')
		if not log_entries or len(log_entries) < 1:
			raise Exception, 'no log data is available.'
	except Exception, e:
		'Error retrieving log data from %s: %s' \
			% (rc.hostname(), str(e))
		return None
	time_now = time()
	entry = ''
	for i in log_entries:
		try:
			log_msg = i.getAttribute('msg')
		except:
			log_msg = ''

		if not log_msg:
			continue

		try:
			log_age = int(i.getAttribute('age'))
		except:
			log_age = 0

		try:
			log_domain = i.getAttribute('domain')
		except:
			log_domain = ''

		try:
			log_pid = i.getAttribute('pid')
		except:
			log_pid = ''

		if log_age:
			entry += ctime(time_now - log_age) + ' '
		if log_domain:
			entry += log_domain
		if log_pid:
			entry += '[' + log_pid + ']'
		entry += ': ' + log_msg + '<br/>'
	return entry

def nodeReboot(rc):
	batch_str = '<module name="reboot"><request sequence="111" API_version="1.0"><function_call name="reboot_now"/></request></module>'

	ricci_xml = rc.batch_run(batch_str)
	return batchAttemptResult(ricci_xml)

def nodeLeaveCluster(rc, cluster_shutdown=False, purge=False):
	cshutdown = 'false'
	if cluster_shutdown == True:
		cshutdown = 'true'

	purge_conf = 'true'
	if purge == False:
		purge_conf = 'false'

	batch_str = '<module name="cluster"><request sequence="111" API_version="1.0"><function_call name="stop_node"><var mutable="false" name="cluster_shutdown" type="boolean" value="' + cshutdown + '"/><var mutable="false" name="purge_conf" type="boolean" value="' + purge_conf + '"/></function_call></request></module>'

	ricci_xml = rc.batch_run(batch_str)
	return batchAttemptResult(ricci_xml)

def nodeFence(rc, nodename):
	batch_str = '<module name="cluster"><request sequence="111" API_version="1.0"><function_call name="fence_node"><var mutable="false" name="nodename" type="string" value="' + nodename + '"/></function_call></request></module>'

	ricci_xml = rc.batch_run(batch_str)
	return batchAttemptResult(ricci_xml)

def nodeJoinCluster(rc, cluster_startup=False):
	cstartup = 'false'
	if cluster_startup == True:
		cstartup = 'true'

	batch_str = '<module name="cluster"><request sequence="111" API_version="1.0"><function_call name="start_node"><var mutable="false" name="cluster_startup" type="boolean" value="' + cstartup + '"/></function_call></request></module>'

	ricci_xml = rc.batch_run(batch_str)
	return batchAttemptResult(ricci_xml)

def startService(rc, servicename, preferrednode=None):
	if preferrednode != None:
		batch_str = '<module name="cluster"><request sequence="1254" API_version="1.0"><function_call name="start_service"><var mutable="false" name="servicename" type="string" value=\"' + servicename + '\"/><var mutable="false" name="nodename" type="string" value=\"' + preferrednode + '\" /></function_call></request></module>'
	else:
		batch_str = '<module name="cluster"><request sequence="1254" API_version="1.0"><function_call name="start_service"><var mutable="false" name="servicename" type="string" value=\"' + servicename + '\"/></function_call></request></module>'

	ricci_xml = rc.batch_run(batch_str)
	return batchAttemptResult(ricci_xml)

def restartService(rc, servicename):
	batch_str = '<module name="cluster"><request sequence="1254" API_version="1.0"><function_call name="restart_service"><var mutable="false" name="servicename" type="string" value=\"' + servicename + '\"/></function_call></request></module>'

	ricci_xml = rc.batch_run(batch_str)
	return batchAttemptResult(ricci_xml)

def stopService(rc, servicename):
	batch_str = '<module name="cluster"><request sequence="1254" API_version="1.0"><function_call name="stop_service"><var mutable="false" name="servicename" type="string" value=\"' + servicename + '\"/></function_call></request></module>'

	ricci_xml = rc.batch_run(batch_str)
	return batchAttemptResult(ricci_xml)

def getDaemonStates(rc, dlist):
	batch_str = '<module name="service"><request API_version="1.0"><function_call name="query"><var mutable="false" name="search" type="list_xml">'

	for item in dlist:
		batch_str += '<service name=\"' + item + '\"/>'

	batch_str += '</var></function_call></request></module>'

	ricci_xml = rc.batch_run(batch_str, async=False)
	if not ricci_xml or not ricci_xml.firstChild:
		luci_log.debug_verbose('no ricci_xml in getDaemonStates')
		return None
	result = extractDaemonInfo(ricci_xml.firstChild)
	return result

def extractDaemonInfo(bt_node):
	if not bt_node:
		return None

	mod_node = None
	for node in bt_node.childNodes:
		if node.nodeType == xml.dom.Node.ELEMENT_NODE:
			if node.nodeName == 'module':
				mod_node = node
	if not mod_node:
		return None

	resp_node = None
	for node in mod_node.childNodes:
		if node.nodeType == xml.dom.Node.ELEMENT_NODE:
			resp_node = node
	if not resp_node:
		return None

	fr_node = None
	for node in resp_node.childNodes:
		if node.nodeType == xml.dom.Node.ELEMENT_NODE:
			fr_node = node
	if not fr_node:
		return None

	varnode = None
	for node in fr_node.childNodes:
		if node.nodeName == 'var':
			varnode = node
			break
	if not varnode:
		return None

	resultlist = list()
	svc_node = None
	for node in varnode.childNodes:
		if node.nodeName == 'service':
			svchash = {}
			try:
				name = node.getAttribute('name')
				if not name:
					raise
			except:
				name = '[unknown]'
			svchash['name'] = name

			try:
				svc_enabled = node.getAttribute('enabled')
			except:
				svc_enabled = '[unknown]'
			svchash['enabled'] = svc_enabled

			try:
				running = node.getAttribute('running')
			except:
				running = '[unknown]'
			svchash['running'] = running
			resultlist.append(svchash)

	return resultlist

def getClusterConf(rc):
	if rc is None:
		return None

	doc = xml.dom.minidom.Document()
	batch = doc.createElement('batch')
	module = doc.createElement('module')
	module.setAttribute('name', 'cluster')
	request = doc.createElement('request')
	request.setAttribute('API_version', '1.0')
	call = doc.createElement('function_call')
	call.setAttribute('name', 'get_cluster.conf')
	request.appendChild(call)
	module.appendChild(request)
	batch.appendChild(module)

	# temporary workaround for ricci bug
	system_info = rc.hostname()
	try:
		rc = RicciCommunicator(system_info)
		if rc is None:
			raise Exception, 'unknown error'
	except Exception, e:
		luci_log.debug_verbose('Ricci error while connecting to %s: %s' \
			% (system_info, str(e)))
		return None
	# end workaround

	try:
		ret = rc.process_batch(batch)
	except Exception, e:
		luci_log.debug_verbose('process_batch error for %s: %s' \
			% (system_info, str(e)))
		return None

	if not ret:
		return None

	cur = ret
	while len(cur.childNodes) > 0:
		for i in cur.childNodes:
			if i.nodeType == xml.dom.Node.ELEMENT_NODE:
				if i.nodeName == 'var' and i.getAttribute('name') == 'cluster.conf':
					return i.childNodes[1].cloneNode(True)
				else:
					cur = i
	return None
